#include "mex.h"
#include "matrix.h"

/* N.B.:  Requires matching gcc and g77 of an older version to work, apparently.   */
/* Verified to work on Suse 10.2 linux with gcc/g77 version 3.4.5                  */
/* (Which was compiled and installed for the purpose (it isn't so hard).)          */
/* Symptoms of a problem are a failure to compile, or a Segmentation Fault on running */

/*  meig.c - Gateway function for eig.f

  function [nr,info,rpath]=meig(zz,rr,ssp,par,bbb,trk)

  zz=depths
  rr=ranges
  ssp=sound speed NzzXNrr
  bbb=bathymetry [range depth]
  trk=tracking array - identified rays to track
  par=vector of parameters

  par(1)=-12;           %  par(1)=angle1
  par(2)=12;            %  par(2)=angle2
  par(3)=4000;          %  par(14)=number of rays
  par(4)=1000;          %  par(3)=source depth
  par(5)=1000;          %  par(4)=receiver depth
  par(6)=3000000;       %  par(5)=source to receiver range

  par(7)=1;             %  par(16)=flag for eigenray (0) or timefront (1)
  par(8)=0;             %  par(17)=flag for ray template (tracking) (0=no,1=yes)
  par(9)=10;            %  par(21)=number of rays in template
  par(10)=0;            %  par(15)=flag for saving ray paths (0=no, 1=yes)
  par(11)=1000;         %  par(6)=range increment to save ray paths
  par(12)=10;           %  par(7)=receiver depth tolerance
  par(13)=150;          %  par(8)=maximum receiver depth tolerance

  par(14)=0;            %  par(9)=travel time offset
  par(15)=1e-7;         %  par(10)=epsilon

  par(16)=0;            %  par(11)=flag for bottom reflections (0=no, 1=yes)
  par(17)=5;            %  par(12)=bathymetry depth tolerance
  par(18)=10;           %  par(13)=number of bottom reflections to allow
  par(19)=0;            %  par(20)=number of ranges in bathymetry
  par(20)=5500      ;   %  par(20)=number of depths in ssp

  par(21)=length(zz);   %  par(21)=number of depths in ssp
  par(22)=length(rr);   %  par(22)=number of ranges in ssp                 */

void  eig_(double *ssp, double *zz, double *rr, double *bb, double *trk, \
           double *s, double *xpar,double *info, double *rpath);

void mexFunction( int nlhs, mxArray *plhs[],
		  int nrhs, const mxArray *prhs[])
{
      int dims0[2],dims1[3];
      double *zz, *rr, *ssp, *bb;
      double *trk, *par, s[4];
      double xpar[25];
      double *nr, *info, *rpath;
/* rpath is 3 X 100 X 25000 = 7.5e6; 3 variables, 100 rays, 25000 points per ray */

/*  The ray information consists of 9 values:
  index, time, init.angle, arr.angle, low.turn,  up.turn,  z rec., s ssp., and number of turning points
  These 9 values are used in the tracking data, and the ray information matrix info.  */

/* The following parameters must be consistent between this program */
/* and the eigenray program to avoid scrambling the arrays.         */
/*    parameter (NZSSP=100,neig=100,npeig=25000,MAXRAYS=15001)      */
/*    parameter (Nprofmax=500,MBATH=5000)                           */

/* par comes in from matlab ; xpar goes out to eigenray */

      if (nrhs != 6) {
          mexErrMsgTxt("meig requires 6 input arguments");
      } else if (nlhs != 3) {
          mexErrMsgTxt("meig requires 3 output argument");
      }
      
/* Store the number of rays that are found                  */
      plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
      nr = mxGetPr(plhs[0]);
  
/* "info" and "rpath" have to be sized the same as in the eig routine. Otherwise   */
/* the data get scrambled in the transfer between this program and eig.f.          */
/* They return to the calling matlab program with these dimensions.                */

/* Ray Information Table */
      dims0[0]=9;
      dims0[1]=25001;     /* MAXRAYS */
      plhs[1] = mxCreateNumericArray(2,dims0,mxDOUBLE_CLASS,mxREAL);
      info = mxGetPr(plhs[1]);
      
/* Ray Path Table - We allow for three columns (r,z,ds/c^2),   */
/* 100 eigenrays, and 25000 points per ray                     */
      dims1[0]=3;        /* rpath parameters:  3,KMAXX,neig  */
      dims1[1]=10000;    /* KMAXX */
      dims1[2]=200;      /* neig */
      plhs[2] = mxCreateNumericArray(3,dims1,mxDOUBLE_CLASS,mxREAL);
      rpath = mxGetPr(plhs[2]); 

/* load depths for ssp */
      zz = mxGetPr(prhs[0]);
/* load ranges for ssp */
      rr = mxGetPr(prhs[1]);
/* load ssp */
      ssp = mxGetPr(prhs[2]);

/* load par */
      par = mxGetPr(prhs[3]);

/* Reassign parameters par->xpar                    */
/* Mainly because we receive back nr[0] = par[21],  */
/* the number of rays found; would rather not add   */
/* another entry in the call to eig for that.       */
/* This could be assigned by a loop, true, but      */
/* here the parameters are also documented          */

/*  angle 1 */
      xpar[0]=par[0];
/*      printf("xpar1=%f\n",par[0]);  
      printf("xpar1=%f\n",xpar[0]);  */
/*  angle 2 */
      xpar[1]=par[1];
/*      printf("xpar2=%f\n",par[1]);  
      printf("xpar2=%f\n",xpar[1]);  */
/*  number of rays to trace */
      xpar[2]=par[2];
/*  src depth */
      xpar[3]=par[3];
/*  rcv depth */
      xpar[4]=par[4];
/*  range */
      xpar[5]=par[5];

/*  flag for eigenrays or timefront */
      xpar[6]=par[6];
/*  flag for template/tracking */
      xpar[7]=par[7];
/*  number of rays in template */
      xpar[8]=par[8];
/*  flag for saving ray paths */
      xpar[9]=par[9];
/*  range increment to save rays */
      xpar[10]=par[10];
/*  rcv depth tolerance */
      xpar[11]=par[11];
/*  max rcv depth tolerance */
      xpar[12]=par[12];

/*  travel time offset */
      xpar[13]=par[13];
/*  epsilon */
      xpar[14]=par[14];

/*  bottom interaction flag */
      xpar[15]=par[15];
/*  bottom interaction tolerance  */
      xpar[16]=par[16];
/*  max bottom bounces */
      xpar[17]=par[17];
/*  number of ranges in bathy */
      xpar[18]=par[18]; 
/*  maximum allowed depth for rays */
      xpar[19]=par[19]; 

/*  number of depths in ssp */
      xpar[20]=par[20];
/*  number of ranges in ssp */
      xpar[21]=par[21];

/*      printf("ipar=%f\n",par[20]); 
      printf("ipar=%f\n",xpar[20]);    */

/* load bathymetry data, if needed */
      bb = mxGetPr(prhs[4]);
     
/* load tracking data, if needed  */
      trk = mxGetPr(prhs[5]);

/* define step sizes - the pre-defined step sizes   */
/* for the Runge-Kutta integration (h in eig.f).    */
      s[0]=1500.0;
      s[1]=10.0;
      s[2]=400.0;
      s[3]=700.0;
      
    eig_(ssp,zz,rr,bb,trk,s,xpar,info,rpath);
/* Returns with info and rpath.         */
/* Also store the number of rays found. */
      nr[0] = xpar[22];

}

